import os.path


class GitSHAMiddleware(object):
    """
    This middleware adds an environ key for the HEAD git SHA1 on repos that it
    is told to watch.
    """
    def get_sha1(self, path):
        git_path = os.path.join(path, '.git')
        with open(os.path.join(git_path, 'HEAD')) as head:
            s = head.read()
            assert s.startswith('ref')
            ref_path = s[5:].strip()
            with open(os.path.join(git_path, ref_path)) as ref:
                s = ref.read()
        return s.strip()

    def __init__(self, app, *watch_paths):
        self.app = app
        self.shas = {}
        for path in watch_paths:
            repo = os.path.basename(path.rstrip('/'))
            try:
                sha = self.get_sha1(path)
            except (IOError, OSError, AssertionError) as e:
                sha = str(e)
            self.shas['git-sha1.%s' % repo] = sha

    def __call__(self, environ, start_response):
        environ.update(self.shas)
        return self.app(environ, start_response)



from webob import Request


class SecureCookiesMiddleware(object):
    """
    This middleware intercepts non-https requests and strips all Set-Cookie
    headers for cookies which are secure.

    Note that this will ONLY work given the following conditions (which are
    true in the typical beaker/pylons/webob environment:
        * the 'secure' attribute comes at the end of the cookie
        * each cookie gets it's own Set-Cookie header
    """
    def __init__(self, app):
        self.app = app

    def __call__(self, environ, start_response):
        req = Request(environ)
        resp = req.get_response(self.app)
        if req.scheme == 'http':
            # Strip all secure cookies.
            if 'Set-Cookie' in resp.headers:
                for header in resp.headers.getall('Set-Cookie'):
                    if header.endswith('secure'):
                        name = header.split('=')[0].strip()
                        resp.unset_cookie(name)
        return resp(environ, start_response)



import logging

from webob import Request
from webob.exc import HTTPTemporaryRedirect

log = logging.getLogger(__name__)


class RenameCookieMiddleware(object):
    """
    This middleware looks for a cookie named ``from_key`` in the request. If
    found, it issues an immediate 302 response that renames the cookie to
    ``to_key``. Other requests are directly passed through.
    """
    def __init__(self, app, from_key, to_key, **kwargs):
        self.app = app
        self.from_key = from_key
        self.to_key = to_key
        self.cookie_kwargs = kwargs

    def __call__(self, environ, start_response):
        req = Request(environ)

        val = req.cookies.get(self.from_key)
        if val:
            log.info("Renaming %s=%s to %s on client IP %s",
                     self.from_key, val, self.to_key, req.remote_addr)
            resp = HTTPTemporaryRedirect(location=req.url)
            resp.delete_cookie(self.from_key)
            resp.set_cookie(self.to_key, val, **self.cookie_kwargs)
        else:
            resp = req.get_response(self.app)

        return resp(environ, start_response)



from webob import Request


class StatusCodeRedirect(object):
    """
    Internally redirects a request based on status code.

    StatusCodeRedirect watches the response of the app it wraps. If the
    response is an error code in the errors sequence passed the request
    will be re-run with the path URL set to the path passed in

    This operation is non-recursive and the output of the second
    request will be used no matter what it is.

    Rewritten from Pylons version, but shares the same semantics.
    """
    def __init__(self, app, errors=(400, 401, 403, 404),
                 path='/error/document'):
        """
        :param errors:
          A sequence (list, tuple) of error code integers that should be
          caught.
        :param path:
          The path to set for the next request down to the application.
        """
        self.app = app
        self.error_path = path
        self.errors = errors

    def __call__(self, environ, start_response):
        req = Request(environ)
        resp = req.get_response(self.app, catch_exc_info=True)

        if resp.status_int in self.errors:
            new_environ = environ.copy()
            new_environ.update({
                'statuscode.original_request': req,
                'statuscode.original_response': resp,
                'PATH_INFO': self.error_path,
            })
            new_req = Request(new_environ)
            new_resp = new_req.get_response(self.app, catch_exc_info=True)
            new_resp.status = resp.status
            resp = new_resp

        return resp(environ, start_response)



from setuptools import setup


setup(name="maitai",
      version='0.2',
      description='Handy WSGI Middleware Utilities',
      long_description='',
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 2.7',
          'Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware',
          'Topic :: Internet :: WWW/HTTP :: HTTP Servers',
      ],
      keywords='wsgi middleware cookies errors',
      url='http://github.com/storborg/maitai',
      author='Scott Torborg',
      author_email='scott@cartlogic.com',
      install_requires=[
          'webob',
          # These are for tests.
          'coverage',
          'nose>=1.1',
          'nose-cover3',
          'webtest',
      ],
      license='MIT',
      packages=['maitai'],
      test_suite='nose.collector',
      tests_require=['nose'],
      zip_safe=False)



from webob import Request
from webob.exc import HTTPTemporaryRedirect


class PruneCookiesMiddleware(object):
    """
    Prune cookies with either a blacklist or a whitelist. If a blacklist is
    provided, delete all cookies from the client which have keys that match the
    sequence provided. If a whitelist is provided, delete all cookies which
    have keys that do not match the sequence provided.
    """
    def __init__(self, app, whitelist=None, blacklist=None):
        self.app = app
        assert (whitelist is None) ^ (blacklist is None), \
            "whitelist or blacklist is required, but not both"
        self.whitelist = whitelist
        self.blacklist = blacklist

    def __call__(self, environ, start_response):
        req = Request(environ)

        to_delete = set()
        for cookie in req.cookies:
            if ((self.whitelist and (cookie not in self.whitelist)) or
                    (self.blacklist and (cookie in self.blacklist))):
                to_delete.add(cookie)

        if to_delete:
            resp = HTTPTemporaryRedirect(location=req.url)
            for cookie in to_delete:
                resp.delete_cookie(cookie)
        else:
            resp = req.get_response(self.app)

        return resp(environ, start_response)
